/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.web.model;

import org.apache.camel.CamelContext;
import org.apache.camel.ServiceStatus;
import org.apache.camel.model.DescriptionDefinition;
import org.apache.camel.model.RouteDefinition;

/**
 * Represents a route.
 * <p/>
 * We need this model to link the RouteDefinition with a CamelContext
 */
public class Route {
    private CamelContext camelContext;
    private RouteDefinition route;

    public Route() {
    }

    public Route(CamelContext camelContext, RouteDefinition route) {
        this.camelContext = camelContext;
        this.route = route;
    }

    public String getId() {
        return route.getId();
    }

    public DescriptionDefinition getDescription() {
        return route.getDescription();
    }

    public ServiceStatus getStatus() {
        return route.getStatus(camelContext);
    }

    public boolean isStartable() {
        return route.isStartable(camelContext);
    }

    public boolean isStoppable() {
        return route.isStoppable(camelContext);
    }
}
